import { diariesLoadingOff, diariesLoadingOn } from "./loading_actions";
import * as DiaryAPIUtil from "../util/diary_api_util";

export const RECEIVE_DIARIES = "RECEIVE_DIARIES";
export const RECEIVE_ONE_DIARY = "RECEIVE_ONE_DIARY";
export const CLEAR_ONE_DIARY = "CLEAR_ONE_DIARY";
export const RECEIVE_DIARY_ERRORS = "RECEIVE_DIARY_ERRORS";

const receiveDiaries = diaries => ({
  type: RECEIVE_DIARIES,
  diaries,
});

const receiveOneDiary = diary => ({
  type: RECEIVE_ONE_DIARY,
  diary,
});

const receiveDiaryErrors = errors => ({
  type: RECEIVE_DIARY_ERRORS,
  errors,
});

const clearOneDiary = diary => ({
  type: CLEAR_ONE_DIARY,
  diary,
});

export const fetchAllDiaries = () => dispatch => {
  dispatch(diariesLoadingOn());
  DiaryAPIUtil.getAllDiaries()
    .then(diaries => {
      dispatch(receiveDiaries(diaries.data));
      dispatch(diariesLoadingOff());
    })
    .catch(err => {
      dispatch(receiveDiaryErrors(err.response.data));
      dispatch(diariesLoadingOff());
    });
};

export const fetchOneDiary = id => dispatch => {
  dispatch(diariesLoadingOn());
  DiaryAPIUtil.getOneDiary(id)
    .then(diary => {
      dispatch(receiveOneDiary(diary.data));
      dispatch(diariesLoadingOff());
    })
    .catch(err => {
      dispatch(receiveDiaryErrors(err.response.data));
      dispatch(diariesLoadingOff());
    });
};

export const createNewDiary = diaryData => dispatch => {
  DiaryAPIUtil.postDiary(diaryData)
    .then(diary => dispatch(receiveOneDiary(diary.data)))
    .catch(err => dispatch(receiveDiaryErrors(err.response.data)));
};

export const deleteDiary = id => dispatch => {
  DiaryAPIUtil.deleteDiary(id)
    .then(diary => dispatch(clearOneDiary(diary.data)))
    .catch(err => dispatch(receiveDiaryErrors(err.response.data)));
};

export const editDiary = (id, diaryData) => dispatch => {
  DiaryAPIUtil.updateDiary(id, diaryData)
    .then(diary => dispatch(receiveOneDiary(diary.data)))
    .catch(err => dispatch(receiveDiaryErrors(err.response.data)));
};
