import { entriesLoadingOff, entriesLoadingOn } from "./loading_actions";
import * as DiaryEntryAPIUtil from "../util/diary_entry_api_util";

export const RECEIVE_ENTRIES = "RECEIVE_ENTRIES";
export const RECEIVE_ONE_ENTRY = "RECEIVE_ONE_ENTRY";
export const CLEAR_ONE_ENTRY = "CLEAR_ONE_ENTRY";
export const RECEIVE_ENTRY_ERRORS = "RECEIVE_ENTRY_ERRORS";

const receiveEntries = entries => ({
  type: RECEIVE_ENTRIES,
  entries,
});

const receiveOneEntry = entry => ({
  type: RECEIVE_ONE_ENTRY,
  entry,
});

const receiveEntryErrors = errors => ({
  type: RECEIVE_ENTRY_ERRORS,
  errors,
});

const clearOneEntry = entry => ({
  type: CLEAR_ONE_ENTRY,
  entry,
});

export const fetchAllEntries = () => dispatch => {
  dispatch(entriesLoadingOn());
  DiaryEntryAPIUtil.getAllEntries()
    .then(entries => {
      dispatch(receiveEntries(entries.data));
      dispatch(entriesLoadingOff());
    })
    .catch(err => {
      dispatch(receiveEntryErrors(err.response.data));
      dispatch(entriesLoadingOff());
    });
};

export const fetchDiaryEntries = diaryId => dispatch => {
  dispatch(entriesLoadingOn());
  DiaryEntryAPIUtil.getDiaryEntries(diaryId)
    .then(entries => {
      dispatch(receiveEntries(entries.data));
      dispatch(entriesLoadingOff());
    })
    .catch(err => {
      dispatch(receiveEntryErrors(err.response.data));
      dispatch(entriesLoadingOff());
    });
};

export const fetchOneEntry = id => dispatch => {
  dispatch(entriesLoadingOn());
  DiaryEntryAPIUtil.getOneEntry(id)
    .then(entry => {
      dispatch(receiveOneEntry(entry.data));
      dispatch(entriesLoadingOff());
    })
    .catch(err => {
      dispatch(receiveEntryErrors(err.response.data));
      dispatch(entriesLoadingOff());
    });
};

export const createNewEntry = (diaryId, entryData) => dispatch => {
  DiaryEntryAPIUtil.postEntry(diaryId, entryData)
    .then(entry => dispatch(receiveOneEntry(entry.data)))
    .catch(err => dispatch(receiveEntryErrors(err.response.data)));
};

export const deleteEntry = id => dispatch => {
  DiaryEntryAPIUtil.deleteEntry(id)
    .then(entry => dispatch(clearOneEntry(entry.data)))
    .catch(err => dispatch(receiveEntryErrors(err.response.data)));
};

export const editEntry = (id, entryData) => dispatch => {
  DiaryEntryAPIUtil.updateEntry(id, entryData)
    .then(entry => dispatch(receiveOneEntry(entry.data)))
    .catch(err => dispatch(receiveEntryErrors(err.response.data)));
};
