import { flashcardsLoadingOff, flashcardsLoadingOn } from "./loading_actions";
import * as FlashcardAPIUtil from "../util/flashcard_api_util";

export const RECEIVE_FLASHCARDS = "RECEIVE_FLASHCARDS";
export const RECEIVE_DUE_FLASHCARD_COUNT = "RECEIVE_DUE_FLASHCARD_COUNT";
export const RECEIVE_DIARY_DUE_FLASHCARD_COUNT =
  "RECEIVE_DIARY_DUE_FLASHCARD_COUNT";
export const RECEIVE_ENTRY_DUE_FLASHCARD_COUNT =
  "RECEIVE_ENTRY_DUE_FLASHCARD_COUNT";
export const RECEIVE_ONE_FLASHCARD = "RECEIVE_ONE_FLASHCARD";
export const CLEAR_ONE_FLASHCARD = "CLEAR_ONE_FLASHCARD";
export const STUDY_ONE_FLASHCARD = "STUDY_ONE_FLASHCARD";
export const RECEIVE_FLASHCARD_ERRORS = "RECEIVE_FLASHCARD_ERRORS";

const receiveFlashcards = flashcards => ({
  type: RECEIVE_FLASHCARDS,
  flashcards,
});

const receiveOneFlashcard = flashcard => ({
  type: RECEIVE_ONE_FLASHCARD,
  flashcard,
});

const receiveFlashcardErrors = errors => ({
  type: RECEIVE_FLASHCARD_ERRORS,
  errors,
});

const clearOneFlashcard = flashcard => ({
  type: CLEAR_ONE_FLASHCARD,
  flashcard,
});

const studyOneFlashcard = flashcard => ({
  type: STUDY_ONE_FLASHCARD,
  flashcard,
});

const receiveDueFlashcardCount = count => ({
  type: RECEIVE_DUE_FLASHCARD_COUNT,
  count,
});

const receiveDiaryDueFlashcardCount = (diaryId, count) => ({
  type: RECEIVE_DIARY_DUE_FLASHCARD_COUNT,
  diaryId,
  count,
});

const receiveEntryDueFlashcardCount = (entryId, count) => ({
  type: RECEIVE_ENTRY_DUE_FLASHCARD_COUNT,
  entryId,
  count,
});

export const fetchAllFlashcards = () => dispatch => {
  dispatch(flashcardsLoadingOn());
  FlashcardAPIUtil.getAllFlashcards()
    .then(flashcards => {
      dispatch(receiveFlashcards(flashcards.data));
      dispatch(flashcardsLoadingOff());
    })
    .catch(err => {
      dispatch(receiveFlashcardErrors(err.response.data));
      dispatch(flashcardsLoadingOff());
    });
};

export const fetchAllDueFlashcards =
  (count = false) =>
  dispatch => {
    if (!count) {
      dispatch(flashcardsLoadingOn());
    }
    FlashcardAPIUtil.getAllDueFlashcards(count)
      .then(flashcards => {
        if (count) {
          dispatch(receiveDueFlashcardCount(flashcards.data));
        } else {
          dispatch(receiveFlashcards(flashcards.data));
          dispatch(flashcardsLoadingOff());
        }
      })
      .catch(err => {
        dispatch(receiveFlashcardErrors(err.response.data));
        dispatch(flashcardsLoadingOff());
      });
  };

export const fetchDiaryFlashcards = diaryId => dispatch => {
  dispatch(flashcardsLoadingOn());
  FlashcardAPIUtil.getDiaryFlashcards(diaryId)
    .then(flashcards => {
      dispatch(receiveFlashcards(flashcards.data));
      dispatch(flashcardsLoadingOff());
    })
    .catch(err => {
      dispatch(receiveFlashcardErrors(err.response.data));
      dispatch(flashcardsLoadingOff());
    });
};

export const fetchEntryFlashcards = entryId => dispatch => {
  dispatch(flashcardsLoadingOn());
  FlashcardAPIUtil.getEntryFlashcards(entryId)
    .then(flashcards => {
      dispatch(receiveFlashcards(flashcards.data));
      dispatch(flashcardsLoadingOff());
    })
    .catch(err => {
      dispatch(receiveFlashcardErrors(err.response.data));
      dispatch(flashcardsLoadingOff());
    });
};
export const fetchDiaryDueFlashcards =
  (diaryId, count = false) =>
  dispatch => {
    dispatch(flashcardsLoadingOn());
    FlashcardAPIUtil.getDiaryDueFlashcards(diaryId, count)
      .then(flashcards => {
        if (count) {
          dispatch(receiveDiaryDueFlashcardCount(diaryId, flashcards.data));
        } else {
          dispatch(receiveFlashcards(flashcards.data));
        }
        dispatch(flashcardsLoadingOff());
      })
      .catch(err => {
        dispatch(receiveFlashcardErrors(err.response.data));
        dispatch(flashcardsLoadingOff());
      });
  };

export const fetchEntryDueFlashcards =
  (entryId, count = false) =>
  dispatch => {
    dispatch(flashcardsLoadingOn());
    FlashcardAPIUtil.getEntryDueFlashcards(entryId, count)
      .then(flashcards => {
        if (count) {
          dispatch(receiveEntryDueFlashcardCount(entryId, flashcards.data));
        } else {
          dispatch(receiveFlashcards(flashcards.data));
        }
        dispatch(flashcardsLoadingOff());
      })
      .catch(err => {
        dispatch(receiveFlashcardErrors(err.response.data));
        dispatch(flashcardsLoadingOff());
      });
  };

export const fetchOneFlashcard = id => dispatch => {
  dispatch(flashcardsLoadingOn());
  FlashcardAPIUtil.getOneFlashcard(id)
    .then(flashcard => {
      dispatch(receiveOneFlashcard(flashcard.data));
      dispatch(flashcardsLoadingOff());
    })
    .catch(err => {
      dispatch(receiveFlashcardErrors(err.response.data));
      dispatch(flashcardsLoadingOff());
    });
};

export const createNewFlashcard = flashcardData => dispatch => {
  FlashcardAPIUtil.postFlashcard(flashcardData)
    .then(flashcard => dispatch(receiveOneFlashcard(flashcard.data)))
    .catch(err => dispatch(receiveFlashcardErrors(err.response.data)));
};

export const deleteFlashcard = id => dispatch => {
  FlashcardAPIUtil.deleteFlashcard(id)
    .then(flashcard => dispatch(clearOneFlashcard(flashcard.data)))
    .catch(err => dispatch(receiveFlashcardErrors(err.response.data)));
};

export const editFlashcard = (id, flashcardData) => dispatch => {
  FlashcardAPIUtil.updateFlashcard(id, flashcardData)
    .then(flashcard => dispatch(receiveOneFlashcard(flashcard.data)))
    .catch(err => dispatch(receiveFlashcardErrors(err.response.data)));
};

export const studyFlashcard = (id, flashcardData) => dispatch => {
  FlashcardAPIUtil.updateFlashcard(id, flashcardData)
    .then(flashcard => {
      dispatch(studyOneFlashcard(flashcard.data))
      dispatch(receiveOneFlashcard(flashcard.data));
    })
    .catch(err => dispatch(receiveFlashcardErrors(err.response.data)));
};
