import React, { useState } from "react";
import { connect } from "react-redux";
import { Route, Switch } from "react-router-dom";
import { AuthRoute, ProtectedRoute } from "../util/route_util";
import ScrollIntoView from "./util/scroll_into_view";
import NavBar from "./nav/navbar";
import SessionFormModalContainer from "./session/session_form_modal_container";
import LoginFormContainer from "./session/login_form_container";
import RegisterFormContainer from "./session/register_form_container";
import HomePage from "./homepage/homepage";
import Footer from "./footer/footer";
import { logout, toggleSessionFormModal } from "../actions/session_actions";
import DiariesIndex from "./diaries/diaries_index";
import StudyDiariesIndex from "./study/study_diaries_index";

const App = props => {
  const [sessionForm, setSessionForm] = useState("login");

  const logoutUser = e => {
    e.preventDefault();
    props.logout();
  };

  const openSessionForm = sessionForm => {
    return e => {
      setSessionForm(sessionForm);
      props.toggleSessionFormModal();
    };
  };

  return (
    <div>
      <ScrollIntoView>
        <NavBar
          loggedIn={props.loggedIn}
          user={props.user}
          openSessionForm={openSessionForm}
          logoutUser={logoutUser}
        />
        <SessionFormModalContainer form={sessionForm} />
        <main>
          <Switch>
            <AuthRoute
              loggedIn={props.loggedIn}
              exact
              path="/login"
              comp={LoginFormContainer}
            />
            <AuthRoute
              loggedIn={props.loggedIn}
              exact
              path="/register"
              comp={RegisterFormContainer}
            />
            <ProtectedRoute
              loggedIn={props.loggedIn}
              path="/diaries"
              comp={DiariesIndex}
            />
            <ProtectedRoute
              loggedIn={props.loggedIn}
              path="/study-menu"
              comp={StudyDiariesIndex}
            />
            <Route
              path="/"
              render={props => (
                <HomePage
                  openSessionForm={openSessionForm}
                  logoutUser={logoutUser}
                  loggedIn={props.loggedIn}
                  user={props.user}
                />
              )}
            />
          </Switch>
        </main>
        <Footer />
      </ScrollIntoView>
    </div>
  );
}

const mapStateToProps = state => ({
  loggedIn: state.session.isAuthenticated,
  user: state.session.user,
});

const mapDispatchToProps = dispatch => ({
  logout: () => dispatch(logout()),
  toggleSessionFormModal: () => dispatch(toggleSessionFormModal()),
});

export default connect(mapStateToProps, mapDispatchToProps)(App);