import React, { Component } from "react";
import { connect } from "react-redux";
import { Switch } from "react-router-dom";
import { deleteDiary, fetchAllDiaries } from "../../actions/diary_actions";
import { ProtectedRoute } from "../../util/route_util";
import DiaryIndexItem from "./diary_index_item";
import EntriesIndex from "./entries_index";
import EntryShow from "./entry_show";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import NewDiaryForm from "./new_diary_form";
import NewEntryForm from "./new_entry_form";

class DiariesIndex extends Component {
  constructor(props) {
    super(props);
    this.state = {
      showNewDiaryForm: false,
      initialLoad: false,
    };
    this.toggleNewDiaryForm = this.toggleNewDiaryForm.bind(this);
    this.handleDeleteDiary = this.handleDeleteDiary.bind(this);
  }

  componentDidMount() {
    const { fetchAllDiaries } = this.props;
    fetchAllDiaries();
    this.setState({ initialLoad: true });
  }

  toggleNewDiaryForm() {
    this.setState({ showNewDiaryForm: !this.state.showNewDiaryForm });
  }

  handleDeleteDiary(diaryId) {
    return e => {
      e.preventDefault();
      this.props.deleteDiary(diaryId);
    };
  }

  render() {
    const {
      diariesLoading,
      entriesLoading,
      stateDiaries,
      user,
    } = this.props;
    const { showNewDiaryForm, initialLoad } = this.state;
    return (
      <div className="diaries-index-container">
        <div className="diaries-index-header">
          <h1 className="section-header">
            <FontAwesomeIcon icon="book" />
            {user.firstName}'s Diaries
          </h1>
        </div>
        <div className="diaries-index-columns">
          {!diariesLoading && initialLoad && (
            <>
              <div className="diaries-list-column">
                <h2>Diaries</h2>
                <button
                  className="big-button"
                  onClick={this.toggleNewDiaryForm}
                >
                  {showNewDiaryForm ? "Cancel" : "Add New Diary"}
                </button>
                {showNewDiaryForm && (
                  <NewDiaryForm showFormToggle={this.toggleNewDiaryForm} />
                )}
                <ul>
                  {Object.values(stateDiaries)
                    .reverse()
                    .map((diary, index) => (
                      <DiaryIndexItem
                        diary={diary}
                        key={`${diary._id}-diary-${index}`}
                        deleteDiary={this.handleDeleteDiary(diary._id)}
                      />
                    ))}
                </ul>
              </div>
              <div className="entries-list-column">
                <h2>Entries</h2>
                <Switch>
                  <ProtectedRoute
                    path="/diaries/:diaryId"
                    comp={EntriesIndex}
                  />
                </Switch>
              </div>
              <div className="entry-column">
                {!entriesLoading && (
                  <Switch>
                    <ProtectedRoute
                      exact
                      path="/diaries/:diaryId/entry/new"
                      comp={NewEntryForm}
                    />
                    <ProtectedRoute
                      path="/diaries/:diaryId/entry/:entryId"
                      comp={EntryShow}
                    />
                  </Switch>
                )}
              </div>
            </>
          )}
        </div>
      </div>
    );
  }
}

const mapStateToProps = state => ({
  stateDiaries: state.entities.diaries,
  diariesLoading: state.loading.diaries,
  entriesLoading: state.loading.entries,
  user: state.session.user,
});

const mapDispatchToProps = dispatch => ({
  fetchAllDiaries: () => dispatch(fetchAllDiaries()),
  deleteDiary: diaryId => dispatch(deleteDiary(diaryId)),
});

export default connect(mapStateToProps, mapDispatchToProps)(DiariesIndex);
