import React, { Component } from "react";
import { connect } from "react-redux";
import { Link } from "react-router-dom";
import {
  deleteEntry,
  fetchDiaryEntries,
} from "../../actions/diary_entry_actions";
import EntryIndexItem from "./entry_index_item";

class EntriesIndex extends Component {
  constructor(props) {
    super(props);
    this.state = {
      showNewEntryForm: false,
      initialLoad: false,
    };
    this.loadDiaryEntries = this.loadDiaryEntries.bind(this);
    this.handleDeleteEntry = this.handleDeleteEntry.bind(this);
  }

  loadDiaryEntries() {
    const { fetchDiaryEntries, diaryId } = this.props;
    fetchDiaryEntries(diaryId);
  }

  componentDidMount() {
    this.loadDiaryEntries();
    this.setState({ initialLoad: true });
  }

  componentDidUpdate(prevProps) {
    if (prevProps.loading && !this.props.loading) {
      this.setState({ initialLoad: true });
    }
    if (prevProps.diaryId !== this.props.diaryId) {
      this.setState({ initialLoad: false }, () => this.loadDiaryEntries());
    }
  }

  toggleNewEntryForm() {
    this.setState({ showNewEntryForm: !this.state.showNewEntryForm });
  }

  handleDeleteEntry(entryId) {
    return e => {
      e.preventDefault();
      this.props.deleteEntry(entryId);
    };
  }

  render() {
    const { loading, stateEntries, diaryId, study, getEntryFlashcardCount } =
      this.props;
    const { initialLoad } = this.state;
    return (
      <ul>
        {loading ? (
          <div>loading</div>
        ) : (
          initialLoad && (
            <>
              {!study && (
                <Link
                  className="big-button"
                  to={`/diaries/${diaryId}/entry/new`}
                >
                  Add New Entry
                </Link>
              )}
              {Object.values(stateEntries)
                .reverse()
                .map((entry, index) => (
                  <EntryIndexItem
                    entry={entry}
                    key={`${entry._id}-entry-${index}`}
                    study={study}
                    deleteEntry={this.handleDeleteEntry(entry._id)}
                    dueFlashcardCount={getEntryFlashcardCount ? getEntryFlashcardCount(entry._id) : null}
                  />
                ))}
            </>
          )
        )}
      </ul>
    );
  }
}

const mapStateToProps = (state, { match }) => ({
  stateEntries: state.entities.entries,
  loading: state.loading.entries,
  user: state.session.user,
  diaryId: match.params.diaryId,
  study: match.path.includes("study-menu"),
});

const mapDispatchToProps = dispatch => ({
  fetchDiaryEntries: diaryId => dispatch(fetchDiaryEntries(diaryId)),
  deleteEntry: entryId => dispatch(deleteEntry(entryId)),
});

export default connect(mapStateToProps, mapDispatchToProps)(EntriesIndex);
