import React, { useState } from "react";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import { Link } from "react-router-dom";
import moment from "moment";

export default function EntryIndexItem({ entry, study, deleteEntry, dueFlashcardCount }) {
  const [showDelete, setShowDelete] = useState(false);
  let link = `/diaries/${entry.diaryId}/entry/${entry._id}`;
  if (study) link = `/study-menu/diary/${entry.diaryId}/entry/${entry._id}`;
  const shortTitle = () => {
    const { title } = entry;
    return title.length < 25 ? title : title.slice(0, 25).trim() + "...";
  };

  return (
    <li className={study && "study-index-item"}>
      <div>
        <Link to={link}>
          <FontAwesomeIcon icon="calendar-alt" />
          <span>
            {entry.title
              ? shortTitle()
              : moment(entry.date).format("D/MMM/YYYY, h:mm a")}
          </span>
        </Link>
        {!study && (
          <button
            className={"delete-button" + (showDelete ? " show" : "")}
            onClick={e => setShowDelete(!showDelete)}
          >
            <FontAwesomeIcon icon="times-circle" />
          </button>
        )}
      </div>
      {showDelete && (
        <div className="delete-div">
          <span>Delete this diary?</span>
          <button className="confirm-button" onClick={deleteEntry}>
            Yes
          </button>
          <button
            className="confirm-button"
            onClick={e => setShowDelete(!showDelete)}
          >
            No
          </button>
        </div>
      )}
      {study && (
        <Link className="big-button" to={link + "/study"}>
          <span>
            Study {dueFlashcardCount ? dueFlashcardCount : ""} Flashcards
          </span>
        </Link>
      )}
    </li>
  );
}
