import React, { useEffect } from "react";
import { Link } from "react-router-dom";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import { fetchAllDueFlashcards } from "../../actions/flashcard_actions";
import { connect } from "react-redux";

const NavBar = props => {
  const {
    fetchFlashcardCount,
    dueFlashcardCount,
    loggedIn,
    user,
    openSessionForm,
    logoutUser,
  } = props;
  useEffect(() => {
    fetchFlashcardCount();
  }, [fetchFlashcardCount, loggedIn]);
  const getLinks = () => {
    return (
      <nav className="nav-menu">
        <span className="nav-resources-menu">
          <Link to={`/diaries`}>Diaries</Link>
          <Link to={`/study-menu`}>
            Study
            {loggedIn && (
              <>
                {" "}
                - <span>{dueFlashcardCount}</span> flashcards due
              </>
            )}
          </Link>
        </span>
        <span className="nav-session-menu">
          {loggedIn ? (
            <>
              <span className="nav-button" to={`/profile`}>
                <FontAwesomeIcon icon="user" />
                <span>Welcome, {user.firstName}</span>
              </span>
              <button className="nav-button" onClick={logoutUser}>
                <FontAwesomeIcon icon="sign-out-alt" />
                <span>Log Out</span>
              </button>
            </>
          ) : (
            <>
              <button
                className="nav-button"
                onClick={openSessionForm("register")}
              >
                <FontAwesomeIcon icon="user-plus" />
                <span>Register</span>
              </button>
              <button className="nav-button" onClick={openSessionForm("login")}>
                <FontAwesomeIcon icon="sign-in-alt" />
                <span>Log In</span>
              </button>
            </>
          )}
        </span>
      </nav>
    );
  };

  return (
    <>
      <header className="site-header">
        <h1 className="site-header-title">
          <Link to={"/"}>SRS Diary</Link>
        </h1>
        {getLinks()}
      </header>
    </>
  );
};

const mapStateToProps = state => ({
  dueFlashcardCount: state.ui.dueFlashcardCount,
  flashcardsLoading: state.loading.flashcards,
  user: state.session.user,
  resourceId: undefined,
});

const mapDispatchToProps = dispatch => ({
  fetchFlashcardCount: () => dispatch(fetchAllDueFlashcards(true)),
});

export default connect(mapStateToProps, mapDispatchToProps)(NavBar);
