import React from "react";
import { Link, withRouter } from "react-router-dom";
import LoadingSpinner from "../loading/loading_spinner";

class RegisterForm extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      email: "",
      username: "",
      password: "",
      confPassword: "",
      firstName: "",
      lastName: "",
      location: "",
    };

    this.update = this.update.bind(this);
    this.renderErrors = this.renderErrors.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  componentDidUpdate(prevProps, prevState) {
    if (this.props.loggedIn && !prevProps.loggedIn) {
      this.props.history.push("/diaries");
    }
  }

  update(field) {
    return e => this.setState({ [field]: e.currentTarget.value });
  }

  handleSubmit(e) {
    e.preventDefault();
    const {
      username,
      email,
      firstName,
      lastName,
      password,
      confPassword,
      birthday
    } = this.state;
    let user = {
      email,
      username,
      firstName,
      lastName,
      password,
      confPassword,
      birthday
    };

    this.props.register(user, this.props.history);
  }

  renderErrors() {
    return (
      <ul>
        {this.props.errors.map((error, i) => (
          <li key={`error-${i}`} className={error.name}>
            {error.message}
          </li>
        ))}
      </ul>
    );
  }

  render() {
    const { loading } = this.props;
    const {
      username,
      email,
      firstName,
      lastName,
      birthday,
      password,
      confPassword,
    } = this.state;
    return (
      <form onSubmit={this.handleSubmit} className="session-form">
        <h1>Register an account</h1>
        <input
          type="email"
          value={email}
          onChange={this.update("email")}
          placeholder="Email"
          required
        />
        <input
          type="text"
          value={username}
          onChange={this.update("username")}
          placeholder="Username"
          required
        />
        <input
          type="password"
          value={password}
          onChange={this.update("password")}
          placeholder="Password"
          required
        />
        <input
          type="password"
          value={confPassword}
          onChange={this.update("confPassword")}
          placeholder="Confirm Password"
          required
        />
        <input
          type="text"
          value={firstName}
          onChange={this.update("firstName")}
          placeholder="First Name"
          required
        />
        <input
          type="text"
          value={lastName}
          onChange={this.update("lastName")}
          placeholder="Last Name"
          required
        />
        <label for="user_birthday">Date of Birth</label>
        <input
          type="date"
          value={birthday}
          onChange={this.update("birthday")}
          placeholder="Birthday"
          id="user_birthday"
        />
        {loading ? (
          <LoadingSpinner />
        ) : (
          <button className="big-button green" type="submit">
            Register new account
          </button>
        )}
        <Link className="big-button" to="/login">
          Log in to existing account
        </Link>
        {this.renderErrors()}
      </form>
    );
  }
}

export default withRouter(RegisterForm);
