import moment from "moment";
import React, { Component } from "react";
import ReactMarkdown from "react-markdown";
import { connect } from "react-redux";
import { Link } from "react-router-dom";
import { studyFlashcard } from "../../actions/flashcard_actions";

export class StudyCard extends Component {
  constructor(props) {
    super(props);

    this.state = {
      context: 0,
      altHeading: null,
      flip: false,
      maxContext: 0,
    };
    this.getMaxContext = this.getMaxContext.bind(this);
    this.cardHeading = this.cardHeading.bind(this);
    this.flipCard = this.flipCard.bind(this);
    this.moreContext = this.moreContext.bind(this);
    this.lessContext = this.lessContext.bind(this);
    this.rememberedCard = this.rememberedCard.bind(this);
    this.forgotCard = this.forgotCard.bind(this);
  }

  componentDidMount() {
    this.setState({
      flip: false,
      context: 0,
      maxContext: this.getMaxContext(),
    });
  }

  componentDidUpdate(prevProps) {
    if (prevProps.card !== this.props.card) {
      this.setState({
        flip: false,
        context: 0,
        maxContext: this.getMaxContext(),
      });
    }
  }

  flipCard() {
    this.setState({ flip: !this.state.flip });
  }

  getMaxContext() {
    const { card } = this.props;
    return card.contextUp.length > card.contextDown.length
      ? card.contextUp.length
      : card.contextDown.length;
  }

  moreContext() {
    if (this.state.context < this.getMaxContext()) {
      this.setState({ context: this.state.context + 1 });
    }
  }

  lessContext() {
    if (this.state.context > 0) {
      this.setState({ context: this.state.context - 1 });
    }
  }

  rememberedCard(e) {
    e.preventDefault();
    const { card, studyFlashcard, nextCard } = this.props;
    const daysSinceDue = moment().diff(moment(card.nextDue), "days");
    const bonusStreak = Math.floor(Math.sqrt(daysSinceDue)) + 1;
    const newStreak = Math.max(bonusStreak, card.correctStreak + 1);
    const nextDue = moment().add(Math.pow(2, newStreak), "days");
    const updatedData = {
      correctStreak: newStreak,
      nextDue: nextDue.toISOString(),
    };
    console.log('updatedData :', updatedData);
    studyFlashcard(card._id, updatedData);
    nextCard();
  }

  forgotCard(e) {
    e.preventDefault();
    const { card, studyFlashcard, nextCard } = this.props;
    const newStreak = 0;
    const nextDue = moment().add(Math.pow(2, newStreak), "days");
    const updatedData = {
      correctStreak: newStreak,
      nextDue: nextDue.toISOString(),
    };
    studyFlashcard(card._id, updatedData);
    nextCard();
  }

  cardHeading() {
    const { card } = this.props;
    let heading = "";
    switch (card.type) {
      case "fill_the_blanks":
        heading = "What is the missing word/phrase?";
        break;
      case "location":
        heading = "Where were you?";
        break;
      case "mood":
        heading = "How did you feel?";
        break;
      default:
        heading = card.heading;
        break;
    }
    return heading;
  }

  render() {
    const { card } = this.props;
    const { context, altHeading, flip, maxContext } = this.state;

    return card ? (
      <div className="study-card">
        <div className="card-body">
          <h2 className="card-heading">{this.cardHeading()}</h2>
          {altHeading && (
            <div className="alt-heading">
              <ReactMarkdown children={altHeading} />
            </div>
          )}
          <div className="contextUp">
            <ReactMarkdown
              children={card.contextUp.slice(0, context).reverse().join(" ")}
            />
          </div>
          <div className="front-side">
            <ReactMarkdown children={card.frontSide} />
          </div>
          {flip && (
            <div className="back-side">
              <h2 className="back-side-header">Answer</h2>
              <ReactMarkdown children={card.backSide} />
            </div>
          )}
          <div className="contextDown">
            <ReactMarkdown
              children={card.contextDown.slice(0, context).join(" ")}
            />
          </div>
        </div>
        <div className="card-interactions">
          {flip ? (
            <ul className="back-menu">
              <li>
                <button className="forgot-btn" onClick={this.forgotCard}>
                  I Forgot
                </button>
              </li>
              <li>
                <button className="flip-btn" onClick={this.flipCard}>
                  Flip
                </button>
              </li>
              <li>
                <button
                  className="remembered-btn"
                  onClick={this.rememberedCard}
                >
                  I Remembered
                </button>
              </li>
              <li>
                <Link
                  className="btn-link"
                  to={`/diaries/${card.diaryId}/entry/${card.diaryEntryId}`}
                >
                  Go to Diary Entry
                </Link>
              </li>
            </ul>
          ) : (
            <ul className="front-menu">
              <li>
                <button onClick={this.lessContext} disabled={context < 1}>
                  Show less context
                </button>
              </li>
              <li>
                <button className="flip-btn" onClick={this.flipCard}>
                  Flip
                </button>
              </li>
              <li>
                <button
                  onClick={this.moreContext}
                  disabled={context >= maxContext}
                >
                  Show more context
                </button>
              </li>
            </ul>
          )}
        </div>
      </div>
    ) : null;
  }
}

const mapDispatchToProps = dispatch => ({
  studyFlashcard: (id, flashcardData) =>
    dispatch(studyFlashcard(id, flashcardData)),
});

export default connect(null, mapDispatchToProps)(StudyCard);
