import React, { Component } from "react";
import StudyCard from "./study_card";

export class StudyDeck extends Component {
  constructor(props) {
    super(props);
    this.state = {
      initialLoad: false,
      currentCard: null,
      finishedCards: false,
      today: new Date(),
      dueCards: []
    };
    this.nextCard = this.nextCard.bind(this);
    this.getDueCards = this.getDueCards.bind(this);
    this.setupDeck = this.setupDeck.bind(this);
  }

  componentDidMount() {
    this.setupDeck();
    this.setState({ initialLoad: true });
  }

  setupDeck() {
    const cards = this.getDueCards().sort(
      // Shuffle cards
      () => Math.random() - 0.5
    );
    this.setState({
      currentCard: cards.shift(),
      dueCards: cards,
    });
  }

  componentDidUpdate(prevProps) {
    if (prevProps.loading !== this.props.loading) {
      this.setState({ initialLoad: true });
    }
    if (prevProps.resourceId !== this.props.resourceId) {
      this.setupDeck();
    }
  }

  getDueCards() {
    const { resourceId, resourceType, stateFlashcards } = this.props;
    const dueCards = Object.values(stateFlashcards).filter(
      ({ nextDue }) => new Date(nextDue) < this.state.today
    );
    console.log("dueCards :", dueCards);
    switch (resourceType) {
      case "diary":
        return dueCards.filter(({ diaryId }) => diaryId === resourceId);
      case "entry":
        return dueCards.filter(
          ({ diaryEntryId }) => diaryEntryId === resourceId
        );
      default:
        return dueCards;
    }
  }

  nextCard() {
    if (this.state.dueCards.length > 0) {
      this.setState({
        currentCard: this.state.dueCards[0],
        dueCards: this.state.dueCards.slice(1)
      });
    } else {
      this.setState({ currentCard: null, finishedCards: true });
    }
  }
  render() {
    const { flashcardsLoading, stateFlashcards } = this.props;
    const { initialLoad, currentCard, dueCards } = this.state;
    return (
      <div className="study-deck-container">
        <h1 className="section-header">Study All Flashcards</h1>
        <h2>{dueCards.length + 1} cards left</h2>
        {!flashcardsLoading &&
          initialLoad &&
          Object.keys(stateFlashcards).length > 0 && (
            <StudyCard
              card={currentCard}
              nextCard={this.nextCard}
            />
          )}
      </div>
    );
  }
}
