import React, { Component } from "react";
import { connect } from "react-redux";
import { Link, Switch } from "react-router-dom";
import { fetchAllDiaries } from "../../actions/diary_actions";
import { ProtectedRoute } from "../../util/route_util";
import DiaryIndexItem from "../diaries/diary_index_item";
import EntriesIndex from "../diaries/entries_index";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import AllCardsDeckContainer from "./all_cards_deck_container";
import DiaryDeckContainer from "./diary_deck_container";
import EntryDeckContainer from "./entry_deck_container";
import { fetchAllDueFlashcards } from "../../actions/flashcard_actions";

class StudyDiariesIndex extends Component {
  constructor(props) {
    super(props);
    this.state = {
      showNewDiaryForm: false,
      initialLoad: false,
    };
    this.getDiaryFlashcardCount = this.getDiaryFlashcardCount.bind(this);
    this.getEntryFlashcardCount = this.getEntryFlashcardCount.bind(this);
    this.getFlashcardCount = this.getFlashcardCount.bind(this);
  }

  componentDidMount() {
    const { fetchAllDiaries, fetchAllDueFlashcards } = this.props;
    fetchAllDiaries();
    fetchAllDueFlashcards();
    this.setState({ initialLoad: true });
  }

  isCardDue({ nextDue }) {
    return new Date(nextDue) < new Date();
  }

  getFlashcardCount() {
    const { stateFlashcards } = this.props;
    return Object.values(stateFlashcards).filter(card => this.isCardDue(card))
      .length;
  }

  getDiaryFlashcardCount(diaryId) {
    const { stateFlashcards } = this.props;
    return Object.values(stateFlashcards).filter(
      card => card.diaryId === diaryId && this.isCardDue(card)
    ).length;
  }

  getEntryFlashcardCount(entryId) {
    const { stateFlashcards } = this.props;
    return Object.values(stateFlashcards).filter(
      card => card.diaryEntryId === entryId && this.isCardDue(card)
    ).length;
  }

  render() {
    const {
      diariesLoading,
      stateDiaries,
      studying,
      flashcardsLoading,
    } = this.props;
    return (
      <>
        <div
          className={"diaries-index-container" + (studying ? " collapse" : "")}
        >
          <div className="diaries-index-header">
            <h1 className="section-header">
              <FontAwesomeIcon icon="book" />
              Study Flashcards
            </h1>
            <Link
              className="big-button study-all-button"
              to="/study-menu/study"
            >
              Study All {this.getFlashcardCount()} Due Flashcards
            </Link>
          </div>
          <div className="diaries-index-columns">
            {!diariesLoading && !flashcardsLoading && (
              <>
                <div className="diaries-list-column">
                  <h2>Diaries</h2>
                  <ul>
                    {Object.values(stateDiaries)
                      .reverse()
                      .map((diary, index) => (
                        <DiaryIndexItem
                          diary={diary}
                          study={true}
                          key={`${diary._id}-diary-${index}`}
                          dueFlashcardCount={this.getDiaryFlashcardCount(
                            diary._id
                          )}
                        />
                      ))}
                  </ul>
                </div>
                <div className="entries-list-column">
                  <h2>Entries</h2>
                  <Switch>
                    <ProtectedRoute
                      path="/study-menu/diary/:diaryId"
                      study={true}
                      thingy={true}
                      getEntryFlashcardCount={this.getEntryFlashcardCount}
                      comp={EntriesIndex}
                    />
                  </Switch>
                </div>
              </>
            )}
          </div>
        </div>
        <div className="study-deck-container">
          <Switch>
            <ProtectedRoute
              path="/study-menu/study"
              study={true}
              comp={AllCardsDeckContainer}
            />
            <ProtectedRoute
              path="/study-menu/diary/:diaryId/study"
              study={true}
              comp={DiaryDeckContainer}
            />
            <ProtectedRoute
              path="/study-menu/diary/:diaryId/entry/:entryId/study"
              study={true}
              comp={EntryDeckContainer}
            />
          </Switch>
        </div>
      </>
    );
  }
}

const mapStateToProps = (state, { location }) => ({
  stateDiaries: state.entities.diaries,
  stateFlashcards: state.entities.flashcards,
  diariesLoading: state.loading.diaries,
  flashcardsLoading: state.loading.flashcards,
  entriesLoading: state.loading.entries,
  user: state.session.user,
  studying: location.pathname.endsWith("/study"),
});

const mapDispatchToProps = dispatch => ({
  fetchAllDiaries: () => dispatch(fetchAllDiaries()),
  fetchAllDueFlashcards: () => dispatch(fetchAllDueFlashcards()),
});

export default connect(mapStateToProps, mapDispatchToProps)(StudyDiariesIndex);
