import {
  RECEIVE_DIARY_DUE_FLASHCARD_COUNT,
  RECEIVE_DUE_FLASHCARD_COUNT,
  RECEIVE_ENTRY_DUE_FLASHCARD_COUNT,
  RECEIVE_FLASHCARDS,
  STUDY_ONE_FLASHCARD,
} from "../actions/flashcard_actions";
import {
  SHOW_SESSION_FORM_MODAL,
  HIDE_SESSION_FORM_MODAL,
  TOGGLE_SESSION_FORM_MODAL,
} from "../actions/session_actions";

const _initialState = {
  showSessionFormModal: false,
  dueFlashcardCount: 0,
  diaryDueFlashcardCount: {},
  entryDueFlashcardCount: {},
};

const UiReducer = (state = _initialState, action) => {
  Object.freeze(state);
  switch (action.type) {
    case SHOW_SESSION_FORM_MODAL:
      return Object.assign({}, state, { showSessionFormModal: true });
    case HIDE_SESSION_FORM_MODAL:
      return Object.assign({}, state, { showSessionFormModal: false });
    case TOGGLE_SESSION_FORM_MODAL:
      return Object.assign({}, state, {
        showSessionFormModal: !state.showSessionFormModal,
      });
    case STUDY_ONE_FLASHCARD:
      return Object.assign({}, state, {
        dueFlashcardCount: state.dueFlashcardCount - 1,
        diaryDueFlashcardCount:
          state.diaryDueFlashcardCount[action.flashcard.diaryId] - 1,
        entryDueFlashcardCount:
          state.entryDueFlashcardCount[action.flashcard.entryId] - 1,
      });
    case RECEIVE_DUE_FLASHCARD_COUNT:
      return Object.assign({}, state, { dueFlashcardCount: action.count });
    case RECEIVE_DIARY_DUE_FLASHCARD_COUNT:
      const newDiaryObject = Object.assign({}, state.diaryDueFlashcardCount, {
        [action.diaryId]: action.count,
      });
      return Object.assign({}, state, {
        diaryDueFlashcardCount: newDiaryObject,
      });
    case RECEIVE_ENTRY_DUE_FLASHCARD_COUNT:
      const newEntryObject = Object.assign({}, state.entryDueFlashcardCount, {
        [action.entryId]: action.count,
      });
      return Object.assign({}, state, {
        entryDueFlashcardCount: newEntryObject,
      });
    case RECEIVE_FLASHCARDS:
      const diaryCounts = Object.assign({}, state.diaryDueFlashcardCount);
      const entryCounts = Object.assign({}, state.entryDueFlashcardCount);
      action.flashcards.forEach(({ diaryId, diaryEntryId }) => {
        diaryCounts[diaryId] =
          diaryId in diaryCounts ? diaryCounts[diaryId] + 1 : 1;
        entryCounts[diaryEntryId] =
          diaryEntryId in entryCounts ? entryCounts[diaryEntryId] + 1 : 1;
      });
      return Object.assign({}, state, {
        diaryDueFlashcardCount: diaryCounts,
        entryDueFlashcardCount: entryCounts,
      });
    default:
      return state;
  }
};

export default UiReducer;
