const mongoose = require("mongoose");
const Diary = require("../models/Diary.model");
const DiaryEntry = require("../models/DiaryEntry.model");
const User = require("../models/User.model");

module.exports = {
  getDiary: async (req, res, next) => {
    try {
      const diary = await Diary.findById(req.params.id);
      if (!diary) return res.status(404).send("Diary not found");
      if (diary.userId.toString() !== req.user._id.toString()) {
        return res
          .status(401)
          .send("You are not authorized to view this diary");
      } else {
        return res.send(diary);
      }
    } catch (error) {
      next(error);
    }
  },
  getUserDiaries: async (req, res, next) => {
    try {
      const diaries = await Diary.find({ userId: req.user._id });
      return res.send(diaries);
    } catch (error) {
      next(error);
    }
  },
  createDiary: async (req, res, next) => {
    try {
      const newDiary = new Diary({
        title: req.body.title,
        userId: req.user._id,
      });
      const savedDiary = await newDiary.save();
      return res.send(savedDiary);
    } catch (error) {
      next(error);
    }
  },
  updateDiary: async (req, res, next) => {
    try {
      const diary = await Diary.findById(req.params.id);
      if (!diary) return res.status(404).send("Diary not found");
      if (req.user._id != diary.userId.toString())
        return res.status(401).send("You are not authorized to do this");

      const updatedDiary = await Diary.findOneAndReplace(
        { _id: diary._id },
        Object.assign(
          {},
          diary.toObject(),
          { ...req.body },
          { updatedAt: Date.now() }
        ),
        { new: true }
      );

      return res.send(updatedDiary);
    } catch (error) {
      next(error);
    }
  },
  deleteDiary: async (req, res, next) => {
    try {
      const diary = await Diary.findById(req.params.id);
      if (!diary) return res.status(404).send("Diary not found");
      if (req.user._id != diary.userId.toString())
        return res.status(401).send("You are not authorized to do this");
      await Flashcard.deleteMany({ diaryId: diary.id });
      await DiaryEntry.deleteMany({diaryId: diary.id});
      const deletedDiary = await Diary.findOneAndDelete({ _id: diary._id });
      return res.send(deletedDiary);
    } catch (error) {
      next(error);
    }
  },
};
