const mongoose = require('mongoose');
const DiaryEntry = require('../models/DiaryEntry.model');
const Flashcard = require('../models/Flashcard.model');

module.exports = {
  getUserFlashcards: async (req, res, next) => {
    try {
      const flashcards = await Flashcard.find({ userId: req.user._id });
      return res.send(flashcards);
    } catch (error) {
      next(error);
    }
  },
  getDueUserFlashcards: async (req, res, next) => {
    try {
      if (req.query.count) {
        const flashcardCount = await Flashcard.countDocuments({
          userId: req.user._id,
          nextDue: { $lte: new Date() },
        });
        return res.json(flashcardCount);
      }
      const flashcards = await Flashcard.find({ userId: req.user._id, nextDue: { $lte: new Date() } });
      return res.send(flashcards);
    } catch (error) {
      next(error);
    }
  },
  getEntryFlashcards: async (req, res, next) => {
    try {
      const flashcards = await Flashcard.find({ diaryEntryId: req.params.id });
      return res.send(flashcards);
    } catch (error) {
      next(error);
    }
  },
  getDiaryFlashcards: async (req, res, next) => {
    try {
      const flashcards = await Flashcard.find({ diaryId: req.params.id });
      return res.send(flashcards);
    } catch (error) {
      next(error);
    }
  },
  getEntryDueFlashcards: async (req, res, next) => {
    try {
      if (req.query.count) {
        const flashcardCount = await Flashcard.countDocuments({
          diaryEntryId: req.params.id,
          nextDue: { $lte: new Date() },
        });
        return res.json(flashcardCount);
      }
      const flashcards = await Flashcard.find({
        diaryEntryId: req.params.id,
        nextDue: { $lte: new Date() },
      });
      return res.send(flashcards);
    } catch (error) {
      next(error);
    }
  },
  getDiaryDueFlashcards: async (req, res, next) => {
    try {
      if (req.query.count) {
        const flashcardCount = await Flashcard.countDocuments({
          diaryId: req.params.id,
          nextDue: { $lte: new Date() },
        });
        return res.json(flashcardCount);
      }
      const flashcards = await Flashcard.find({
        diaryId: req.params.id,
        nextDue: { $lte: new Date() },
      });
      return res.send(flashcards);
    } catch (error) {
      next(error);
    }
  },
  getFlashcard: async (req, res, next) => {
    try {
      const flashcard = await Flashcard.findById(req.params.id);
      if (!flashcard) return res.status(404).send("Flashcard not found");
      if (flashcard.userId.toString() !== req.user._id.toString()) {
        return res
          .status(401)
          .send("You are not authorized to view this flashcard");
      } else {
        return res.send(flashcard);
      }
    } catch (error) {
      next(error);
    }
  },
  createFlashcard: async (req, res, next) => {
    try {
      const diaryEntry = await DiaryEntry.findById(req.params.id);
      if (!diaryEntry) return res.status(404).send("Diary entry not found");
      const newFlashcard = new Flashcard({
        ...req.body,
        userId: req.user._id,
        diaryEntryId: diaryEntry._id,
        diaryId: diaryEntry.diaryId,
        entryDate: diaryEntry.date
      });
      const savedFlashcard = await newFlashcard.save();

      return res.send(savedFlashcard);
    } catch (error) {
      next(error);
    }
  },
  updateFlashcard: async (req, res, next) => {
    try {
      const flashcard = await Flashcard.findById(req.params.id);
      if (!flashcard) return res.status(404).send("Flashcard not found");
      if (flashcard.userId.toString() !== req.user._id.toString())
        return res.status(401).send("You are not authorized to do this");

      const updatedFlashcard = await Flashcard.findOneAndReplace(
        { _id: flashcard._id },
        Object.assign(
          {},
          flashcard.toObject(),
          { ...req.body },
          { updatedAt: Date.now() }
        ),
        { new: true }
      );

      return res.send(updatedFlashcard);
    } catch (error) {
      next(error);
    }
  },
  deleteFlashcard: async (req, res, next) => {
    try {
      const flashcard = await Flashcard.findById(req.params.id);
      if (!flashcard) return res.status(404).send("Flashcard not found");
      if (flashcard.userId.toString() !== req.user._id.toString())
        return res.status(401).send("You are not authorized to do this");

      const deletedFlashcard = await Flashcard.findByIdAndDelete(flashcard._id);
      return res.send(deletedFlashcard);
    } catch (error) {
      next(error);
    }
  },
};