const express = require("express");
const diaryRouter = express.Router();

const passport = require("passport");
const DiaryController = require("../controllers/diary.controller");
const DiaryEntryController = require("../controllers/diaryEntry.controller");
const FlashcardController = require("../controllers/flashcard.controller");
const {
  newDiaryValidationSchema,
  diaryValidationSchema,
} = require("../validation/diary.joiSchema");
const { newDiaryEntryValidationSchema } = require("../validation/diaryEntry.joiSchema");
const joiValidator = require("express-joi-validation").createValidator({
  passError: true,
});

// GET logged in user's diaries
diaryRouter.get(
  "/",
  passport.authenticate("jwt", { session: false }),
  DiaryController.getUserDiaries
);
// GET one diary by ID
diaryRouter.get(
  "/:id",
  passport.authenticate("jwt", { session: false }),
  DiaryController.getDiary
);
// POST a new diary
diaryRouter.post(
  "/",
  passport.authenticate("jwt", { session: false }),
  joiValidator.body(newDiaryValidationSchema),
  DiaryController.createDiary
);
// PUT updated info for a diary
diaryRouter.put(
  "/:id",
  passport.authenticate("jwt", { session: false }),
  joiValidator.body(diaryValidationSchema),
  DiaryController.updateDiary
);
// DELETE a diary
diaryRouter.delete(
  "/:id",
  passport.authenticate("jwt", {session: false}),
  DiaryController.deleteDiary
)
// POST a new diary entry
diaryRouter.post(
  "/:id/newentry",
  passport.authenticate("jwt", {session: false}),
  joiValidator.body(newDiaryEntryValidationSchema),
  DiaryEntryController.createEntry
)
// GET entries from a diary
diaryRouter.get(
  "/:id/entries",
  passport.authenticate("jwt", { session: false }),
  DiaryEntryController.getDiaryEntries
);
// GET flashcards from a diary
diaryRouter.get(
  "/:id/flashcards",
  passport.authenticate("jwt", { session: false }),
  FlashcardController.getDiaryFlashcards
);
// GET due flashcards from a diary
diaryRouter.get(
  "/:id/flashcards/due",
  passport.authenticate("jwt", { session: false }),
  FlashcardController.getDiaryDueFlashcards
);

module.exports = diaryRouter;
