const express = require("express");
const diaryEntryRouter = express.Router();

const passport = require("passport");
const DiaryEntryController = require("../controllers/diaryEntry.controller");
const FlashcardController = require("../controllers/flashcard.controller");
const {
  diaryEntryValidationSchema,
} = require("../validation/diaryEntry.joiSchema");
const { newFlashcardValidationSchema } = require("../validation/flashcard.joiSchema");
const joiValidator = require("express-joi-validation").createValidator({
  passError: true,
});

// GET logged in user's diary entries
diaryEntryRouter.get(
  "/",
  passport.authenticate("jwt", { session: false }),
  DiaryEntryController.getUserEntries
);
// GET diary entry if logged in
diaryEntryRouter.get(
  "/:id",
  passport.authenticate("jwt", { session: false }),
  DiaryEntryController.getEntry
);
// POST new entry is in Diary Router
// PUT updated entry information
diaryEntryRouter.put(
  "/:id",
  passport.authenticate("jwt", { session: false }),
  joiValidator.body(diaryEntryValidationSchema),
  DiaryEntryController.updateEntry
);
// DELETE a diary entry
diaryEntryRouter.delete(
  "/:id",
  passport.authenticate("jwt", {session: false}),
  DiaryEntryController.deleteEntry
)

// POST a new flashcard
diaryEntryRouter.post(
  "/:id/newflashcard",
  passport.authenticate("jwt", {session: false}),
  joiValidator.body(newFlashcardValidationSchema),
  FlashcardController.createFlashcard
)
// GET flashcards for one diary entry
diaryEntryRouter.get(
  "/:id/flashcards",
  passport.authenticate("jwt", { session: false }),
  FlashcardController.getEntryFlashcards
);
// GET due flashcards for one diary entry
diaryEntryRouter.get(
  "/:id/flashcards/due",
  passport.authenticate("jwt", { session: false }),
  FlashcardController.getEntryFlashcards
);
module.exports = diaryEntryRouter;
