const express = require("express");
const flashcardRouter = express.Router();

const passport = require("passport");
const FlashcardController = require("../controllers/flashcard.controller");
const {
  flashcardValidationSchema,
} = require("../validation/flashcard.joiSchema");
const joiValidator = require("express-joi-validation").createValidator({
  passError: true,
});

// GET logged in user's flashcards
flashcardRouter.get(
  "/",
  passport.authenticate("jwt", { session: false }),
  FlashcardController.getUserFlashcards
);
// GET logged in user's due flashcards
flashcardRouter.get(
  "/due",
  passport.authenticate("jwt", { session: false }),
  FlashcardController.getDueUserFlashcards
);
// GET a single flashcard, with auth
flashcardRouter.get(
  "/:id",
  passport.authenticate("jwt", { session: false }),
  FlashcardController.getFlashcard
);
// POST new flashcard is in DiaryEntry Router
// PUT updated entry information
flashcardRouter.put(
  "/:id",
  passport.authenticate("jwt", { session: false }),
  joiValidator.body(flashcardValidationSchema),
  FlashcardController.updateFlashcard
);
// DELETE a diary entry
flashcardRouter.delete(
  "/:id",
  passport.authenticate("jwt", { session: false }),
  FlashcardController.deleteFlashcard
);

module.exports = flashcardRouter;
