const express = require("express");
const userRouter = express.Router();
const bcryptjs = require("bcryptjs");
const jsonwebtoken = require("jsonwebtoken");

const passport = require("passport");
const User = require("../models/User.model");
const UserController = require("../controllers/user.controller");
const {
  userValidationSchema,
  loginValidationSchema,
  registerUserValidationSchema,
} = require("../validation/user.joiSchema");
const joiValidator = require("express-joi-validation").createValidator({
  passError: true,
});

// GET currently logged in user
userRouter.get(
  "/current",
  passport.authenticate("jwt", { session: false }),
  UserController.getCurrentlyLoggedInUser
);
// GET one user
userRouter.get("/:id", UserController.getUserById);
// POST one new user
userRouter.post(
  "/register",
  joiValidator.body(registerUserValidationSchema),
  UserController.postUser
);
// POST credentials and login a user
userRouter.post(
  "/login",
  joiValidator.body(loginValidationSchema),
  UserController.loginUser
);
// PUT updated user information
userRouter.put(
  "/:id",
  passport.authenticate("jwt", { session: false }),
  joiValidator.body(userValidationSchema),
  UserController.updateUserInfo
);

module.exports = userRouter;
