const faker = require("faker");

const emphasiseWords = wordArray => {
  for (let i = 0; i < wordArray.length; i++) {
    const word = wordArray[i];
    if (word.length > 4 && faker.datatype.number({ min: 1, max: 5 }) > 4) {
      const emphasis = faker.datatype.boolean() ? "*" : "**";
      if (word.startsWith("\n")) {
        wordArray[i] = "\n" + emphasis + word.slice(2) + emphasis;
      } else if (word.endsWith("\n")) {
        wordArray[i] =
          emphasis + word.slice(0, word.length - 2) + emphasis + "\n";
      } else {
        wordArray[i] = emphasis + word + emphasis;
      }
    }
  }
  return wordArray;
};

const markdownElementGenerators = {
  paragraph: function () {
    const rawWords = faker.lorem
      .paragraphs(faker.datatype.number({ min: 1, max: 5 }), " \n")
      .split(" ");
    return emphasiseWords(rawWords).join(" ") + "\n";
  },
  list: function () {
    let output = "";
    if (faker.datatype.number({ min: 1, max: 5 }) > 3) {
      output +=
        "## " +
        faker.lorem.sentence(faker.datatype.number({ min: 2, max: 7 })) +
        "\n";
    }

    for (let i = 0; i < faker.datatype.number({ min: 2, max: 5 }); i++) {
      output +=
        "* " +
        emphasiseWords(
          faker.lorem
            .sentences(faker.datatype.number({ min: 1, max: 3 }))
            .split(" ")
        ).join(" ") +
        "\n";
    }

    return output;
  },
  subheading: function (type = "h1") {
    let output = "";
    switch (type) {
      case "h1":
        output += "# ";
        break;
      case "h2":
        output += "## ";
        break;
      case "h3":
        output += "### ";
        break;
      case "h4":
        output += "#### ";
        break;
      default:
        break;
    }
    output +=
      faker.lorem.sentence(faker.datatype.number({ min: 2, max: 7 })) + "\n";
    return output;
  },
};

module.exports = markdownElementGenerators;
