const mongoose = require("mongoose");
const express = require("express");
const faker = require("faker");
const bcryptjs = require("bcryptjs");
const { validUsers, testUser } = require("../test/datasets/user.testdata");
const { validDiaries } = require("../test/datasets/diary.testdata");
const Diary = require("../models/Diary.model");
const { validDiaryEntries } = require("../test/datasets/diaryentry.testdata");
const DiaryEntry = require("../models/DiaryEntry.model");
const { runCardmakerService } = require("../services/index.services");
const Flashcard = require("../models/Flashcard.model");
const seedRouter = express.Router();

seedRouter.post("/all", async (req, res, next) => {
  try {
    await mongoose.connection.dropDatabase();
    const userDocs = [];
    userDocs.push(
      testUser
      // , ...validUsers.slice(0, 2)
    );
    const hashedUserDocs = userDocs.map(async user => {
      const hashSalt = await bcryptjs.genSalt(10);
      const passwordHash = await bcryptjs.hash(user.password, hashSalt);
      user.password = passwordHash;
      return user;
    });
    console.log(`hashedUserDocs`, hashedUserDocs);
    const users = await User.create(...(await Promise.all(hashedUserDocs)));

    const diaryDocuments = [];
    for (const user of [users]) {
      const userDiaries = faker.random
        .arrayElements(validDiaries, faker.datatype.number({ min: 1, max: 3 }))
        .map(x => {
          return { ...x, userId: user._id };
        });
      diaryDocuments.push(...userDiaries);
    }
    const diaries = await Diary.create(...diaryDocuments);

    const entryDocuments = [];
    for (const diary of diaries) {
      const diaryEntries = faker.random
        .arrayElements(
          validDiaryEntries,
          faker.datatype.number({ min: 3, max: 10 })
        )
        .map(x => {
          return {
            ...x,
            userId: diary.userId,
            diaryId: diary._id,
          };
        });
      entryDocuments.push(...diaryEntries);
    }
    const entries = await DiaryEntry.create(...entryDocuments);
    const flashcards = [];
    for (const user of [users]) {
      const myEntries = await DiaryEntry.find({ userId: user._id });
      const flashcardDocs = await runCardmakerService({
        user: Object.assign({}, user._doc),
        entries: myEntries,
      });
      const myCards = await Flashcard.create(flashcardDocs.cardData);
      flashcards.push(...myCards);
    }

    const result = `Seeded ${[users].length} users, ${diaries.length} diaries, ${entries.length} diary entries, and ${flashcards.length} flashcards`;
    console.log(result);
    return res.send(result);
  } catch (error) {
    next(error);
  }
});

module.exports = seedRouter;
