const mongoose = require("mongoose");
const { Worker } = require("worker_threads");
const DiaryEntryModel = require("../models/DiaryEntry.model");

function runCardmakerService(diaryEntries) {
  const sanitisedEntries = diaryEntries.entries.map(x => {
    return Object.assign({}, x._doc, {
      userId: x.userId.toString(),
      _id: x._id.toString(),
      diaryId: x.diaryId.toString(),
    });
  });

  return new Promise((resolve, reject) => {
    const worker = new Worker("./services/cardmaker.service.js", {
      workerData: { diaryEntries: sanitisedEntries, user: diaryEntries.user },
    });
    worker.on("message", resolve);
    worker.on("error", reject);
    worker.on("exit", code => {
      if (code !== 0)
        reject(
          new Error(`Cardmaker service worker stopped with error code ${code}`)
        );
    });
  });
}


module.exports = {
  runCardmakerService,
};
