const User = require("../../models/User.model");
const Diary = require("../../models/Diary.model");
const { testUser } = require("../datasets/user.testdata");
const { validDiaries } = require("../datasets/diary.testdata");
const { expect } = require("chai");

describe('Diary model', () => {
  beforeEach(async function () {
    const user = await new User(testUser).save();
    this.currentTest.user = user;
  })

  it("should create new diary with valid data", async function () {
    const diaryData = validDiaries[0];
    const diary = new Diary({
      ...diaryData,
      userId: this.test.user._id
    })
    const savedDiary = await diary.save();
    expect(savedDiary).to.equal(diary);
    expect(savedDiary).to.have.property("_id");
  });

  it("should fail to create new diary with no userId", async function () {
    const diaryData = validDiaries[0];
    const diary = new Diary({
      ...diaryData,
    })
    try {
      await diary.save();
      throw new Error("Should not have saved this record!");
    } catch (err) {
      expect(err.message).to.include("validation failed");
    }
  });
  
})
