const User = require("../../models/User.model");
const Diary = require("../../models/Diary.model");
const { testUser } = require("../datasets/user.testdata");
const { validDiaries } = require("../datasets/diary.testdata");
const { expect } = require("chai");
const { validDiaryEntries } = require("../datasets/diaryentry.testdata");
const DiaryEntry = require("../../models/DiaryEntry.model");

describe("DiaryEntry model", () => {
  beforeEach(async function () {
    const user = await new User(testUser).save();
    this.currentTest.user = user;
    const diaryData = validDiaries[0];
    const diary = await new Diary({
      ...diaryData,
      userId: this.currentTest.user._id,
    }).save();
    this.currentTest.diary = diary;
  });

  it("should create new entry with valid data", async function () {
    const entryData = validDiaryEntries[0];
    const entry = new DiaryEntry({
      ...entryData,
      userId: this.test.user._id,
      diaryId: this.test.diary._id
    })
    const savedEntry = await entry.save();
    expect(savedEntry).to.equal(entry);
    expect(savedEntry).to.have.property("_id");
  });

  it("should fail to create new entry with no userId", async function () {
    const entryData = validDiaryEntries[0];
    const entry = new DiaryEntry({
      ...entryData,
      diaryId: this.test.diary._id
    })

    try {
      await entry.save();
      throw new Error("Should not have saved this record!");
    } catch (err) {
      expect(err.message).to.include("validation failed");
    }
  });

  it("should fail to create new entry with no diaryId", async function () {
    const entryData = validDiaryEntries[0];
    const entry = new DiaryEntry({
      ...entryData,
      userId: this.test.user._id
    })

    try {
      await entry.save();
      throw new Error("Should not have saved this record!");
    } catch (err) {
      expect(err.message).to.include("validation failed");
    }
  });

});
