const Joi = require("joi");
Joi.objectId = require("joi-objectid")(Joi);

const userValidationSchema = Joi.object({
  username: Joi.string()
    .alphanum()
    .min(3)
    .max(30)
    .label("Username")
    .alter({ new: schema => schema.required() }),
  email: Joi.string()
    .email()
    .label("Email")
    .alter({ new: schema => schema.required() }),
  firstName: Joi.string()
    .alphanum()
    .label("First name")
    .alter({ new: schema => schema.required() }),
  lastName: Joi.string()
    .alphanum()
    .label("Last name")
    .alter({ new: schema => schema.required() }),
  password: Joi.string()
    .label("Password")
    .min(6)
    .alter({ new: schema => schema.required() }),
  confPassword: Joi.string()
    .valid(Joi.ref("password"))
    .messages({ "any.only": "Passwords must match" })
    .label("Confirm password")
    .alter({ new: schema => schema.required() }),
  birthday: Joi.date().less("now").label("Birthday"),
});

module.exports.userValidationSchema = userValidationSchema;

module.exports.loginValidationSchema = Joi.object({
  email: Joi.string().email().label("Email").required(),
  password: Joi.string().alphanum().label("Password").required(),
});

module.exports.registerUserValidationSchema = userValidationSchema.tailor("new");